/* indent:4  tabsize:8  font:fixed-width */

#import "SeeheavenGameDelegate.h"
#import "SeeheavenSuitDelegate.h"
#import "../Solitaire/Solitaire.h"
#import "Seeheaven.h"
#import "SeeheavenUndoList.h"

@implementation SeeheavenGameDelegate

- setSuitCardPileViews:(id [])theViews
{
    int i;

    for (i = 0; i < 4; i++)
        suitCardPileViews[i] = *(theViews + i);

    return self;
}


- setTempCardPileViews:(id [])theViews
{
    int i;

    for (i = 0; i < 4; i++)
        tempCardPileViews[i] = *(theViews + i);

    return self;
}


- clickedCard:aCard in:aCardPileView
{
    id cardPile = [aCardPileView cardPile];

    if ((aCard == [cardPile cardAt:CS_TOP]) && (![aCard isFaceUp]))
    {
        [aCard flip];
        [SolGameController() logFlipCard: aCard in: aCardPileView];
        [SolGameController() checkForWin];
        [SolGameController() logCheck];
        [aCardPileView display];
    }
    return self;
}


- doubleClickedCard:aCard in:aCardPileView
{
    int i;

    id cardPile = [aCardPileView cardPile];

    if ((aCard == [cardPile cardAt:CS_TOP]) && (![aCard isFaceUp]))
    {
        [aCard flip];
        [SolGameController() logFlipCard: aCard in: aCardPileView];
        [aCardPileView display];
	[SolGameController() checkForWin];
        [SolGameController() logCheck];
    }
    else if (aCard == [cardPile cardAt:CS_TOP])
    {
        for (i = 0; i < 4; i++)
        {
            [[suitCardPileViews[i] delegate]setDoubleClickCheck];
            if ([[suitCardPileViews[i] delegate] canAcceptPile:cardPile
	                                       from:aCardPileView
	                                         in:suitCardPileViews[i]])
            {
                [[suitCardPileViews[i] cardPile]
		                   addCard:[cardPile cardAt:CS_TOP]];
		[SolGameController() logAddCard: aCard in: suitCardPileViews[i]];
                [cardPile removeCard:[cardPile cardAt:CS_TOP]];
		[SolGameController() logRemoveCard: aCard in: aCardPileView];
                [aCardPileView display];
                [suitCardPileViews[i] display];

		[SolGameController() checkForWin];
		[SolGameController() logCheck];
                return self;
            }
        }
    }
    return self;
}


- (BOOL)draggedPile:aCardPile from:aCardPileView
{
    id cardPile = [aCardPileView cardPile];
    id firstCard = [aCardPile cardAt:CS_TOP];
    int cardIndex = [cardPile cardIndex:firstCard];
    int i;
    int total;
    id newCard;
    CardSuit suit, lastSuit;
    CardValue lastValue;
    int freePiles;

    if ([firstCard isFaceUp])
    {
        total = [cardPile cardCount];
        lastValue = [firstCard value];
	lastSuit = [firstCard suit];

	/* Can only drag if all cards above are same suit and in
	   descending sequence */
        for (i = cardIndex + 1; i < total; i++)
	{
	    newCard = [cardPile cardAt:i];
            if ([newCard value] != --lastValue) return NO;
	    suit = [newCard suit];
	    if (suit != lastSuit) return NO;
	    lastSuit = suit;
	}

        /* Count number of empty temporary piles.  */
        for (i = freePiles = 0; i < 4; i++)
            if ([[tempCardPileViews[i] cardPile] cardCount] == 0)
                freePiles++;

        /* Move cards above dragged card onto temporary pile */
        while (++cardIndex < [cardPile cardCount] && freePiles-- > 0)
        {
            [aCardPile addCard:[cardPile cardAt:cardIndex]];
        }
	if (cardIndex == [cardPile cardCount])
	  return YES;
    }
    return NO;
}


- (BOOL)canAcceptPile:aCardPile from:sender in:aCardPileView
{
    id droppingCard = [aCardPile cardAt:CS_BOTTOM];
    id targetCard = [[aCardPileView cardPile] cardAt:CS_TOP];

    if (targetCard)
    {
        if (([targetCard isFaceUp]) &&
                ([targetCard suit] == [droppingCard suit]) &&
                ([targetCard value] == [droppingCard value] + 1))
        {
            return YES;
        }
    }
    else
    {
        if ([droppingCard value] == CS_KING)
        {
            return YES;
        }
    }
    return NO;
}


- removedPile:aCardPile from:aCardPileView
{
    int i;
    for(i=[aCardPile cardCount]-1;i>=0;i--) {
	[SolGameController() logRemoveCard: [aCardPile cardAt: i]
            in: aCardPileView];
    }
    [SolGameController() checkForWin];
    [SolGameController() logCheck];
    return self;
}

- acceptPile:aCardPile in:aCardPileView
{
    int i;
    for(i=0;i< [aCardPile cardCount];i++) {
	[SolGameController() logAddCard: [aCardPile cardAt: i]
            in: aCardPileView];
    }
    /* logCheck on removal only */
    return self;
}


- getOffset:(float *) xOffset :(float *) yOffset forSize:(CardSize)aSize
{
    *xOffset = 0.0;
    if (aSize == CS_SMALL)
        *yOffset = 25.0;
    else
        *yOffset = 25.0;

    return self;
}

@end
