/* indent:4  tabsize:8  font:fixed-width */

/*---------------------------------------------------------------------------
|----------------------------------------------------------------------------
|
|    Seeheaven:Undo list
|
|    Seeheaven Undo module.
|
|    CREDITS
|
|    Shinji Kono <kono@ie.u-ryukyu.ac.jp>
|        -- design, implementation, documentation
|
|----------------------------------------------------------------------------
\---------------------------------------------------------------------------*/

#import "SeeheavenUndoList.h"
#import "../Solitaire/CardSet.subproj/cardset.h"

@implementation SeeheavenChange

- set: aCard in: aCardPileView as: (change_type) t {
    changer = aCardPileView;
    card = aCard;
    type = t;
    return self;
} 


- (change_type) doChange {
    switch (type) {
	case C_ADD:    
	    [[changer cardPile] addCard: card];  
	    break; 
	case C_REMOVE:   
	    [[changer cardPile] removeCard: card];     
	    break; 
	case C_FLIP:     
	    [card flip];        
	    break; 
	default:     
	    return C_COMMIT;
    }
    [changer display];
    return C_OK; 
} 

- (change_type) undoChange {
    switch (type) {
	case C_ADD:    
	    [[changer cardPile] removeCard: card];  
	    break; 
	case C_REMOVE:   
	    [[changer cardPile] addCard: card];     
	    break; 
	case C_FLIP:     
	    [card flip];        
	    break; 
	default:     
	    return C_COMMIT;
    }
    [changer display];
    return C_OK; 
} 
@end

@implementation SeeheavenUndoList

- clear {
    current = 0;
    last = 0;
    return self;
} 
- reset {
    current = 0;
    return self;
} 

- freeObjects {
    [super freeObjects];
    [self clear];
    return self;
} 

- add: aCard in: aCardView as: (change_type) t {
    if(current >= [self count]) {
	[self addObject: [
           [SeeheavenChange new] set: aCard in: aCardView as: t]];
    } else {
       [[self objectAt: current] set: aCard in: aCardView as: t];
    }
    last = ++current;
    return self;
}

- logAddCard: aCard in: aCardView {
    [self add: aCard in: aCardView as: C_ADD];
    return self;
} 
- logRemoveCard: aCard in: aCardView {
    [self add: aCard in: aCardView as: C_REMOVE];
    return self;
} 
- logFlipCard: aCard in: aCardView {
    [self add: aCard in: aCardView as: C_FLIP];
    return self;
} 
- logCheck {
    [self add: NULL in: NULL as: C_COMMIT];
    return self;
} 

- undo {
    if(current>0) {
	[[self objectAt: --current] undoChange];
	while(current>0 &&
	     [[self objectAt: --current] undoChange]!=C_COMMIT);
    }
    return self;
} 

- redo {
    if(current<last) {
	[[self objectAt: current++] doChange];
	while(current<last &&
	     [[self objectAt: current++] doChange]!=C_COMMIT);
    }
    return self;
} 

@end
